@@ -74,6 +74,9 @@
 /* Define to the type of arg 7 for getnameinfo. */
 /* #undef GETNAMEINFO_TYPE_ARG7 */
 
+/* Define to 1 if you have the alarm function. */
+#define HAVE_ALARM 1
+
 /* Define to 1 if you have the <arpa/inet.h> header file. */
 #define HAVE_ARPA_INET_H 1
 
@@ -104,12 +107,6 @@
 /* disabled non-blocking sockets */
 /* #undef HAVE_DISABLED_NONBLOCKING */
 
-/* Define to 1 if you have the <dlfcn.h> header file. */
-#define HAVE_DLFCN_H 1
-
-/* Define to 1 if you have the `dlopen' function. */
-#define HAVE_DLOPEN 1
-
 /* Define to 1 if you have the `ENGINE_load_builtin_engines' function. */
 /* #undef HAVE_ENGINE_LOAD_BUILTIN_ENGINES */
 #define HAVE_ENGINE_LOAD_BUILTIN_ENGINES 1
@@ -211,16 +208,10 @@
 /* Define to 1 if you have the `inet_addr' function. */
 #define HAVE_INET_ADDR 1
 
-/* Define to 1 if you have the `inet_ntoa' function. */
-#define HAVE_INET_NTOA 1
+/* Define to 1 if you have a IPv6 capable working inet_ntop function. */
+/* #undef HAVE_INET_NTOP */
 
-/* Define to 1 if you have the `inet_ntoa_r' function. */
-/* #undef HAVE_INET_NTOA_R */
-
-/* inet_ntoa_r() is declared */
-/* #undef HAVE_INET_NTOA_R_DECL */
-
-/* Define to 1 if you have the `inet_pton' function. */
+/* Define to 1 if you have a IPv6 capable working inet_pton function. */
 /* #undef HAVE_INET_PTON */
 
 /* Define to 1 if you have the <inttypes.h> header file. */
@@ -244,9 +235,6 @@
 /* Define to 1 if you have the <krb.h> header file. */
 /* #undef HAVE_KRB_H */
 
-/* Define to 1 if you have the `dl' library (-ldl). */
-#define HAVE_LIBDL 1
-
 /* Define to 1 if you have the <libgen.h> header file. */
 /* #undef HAVE_LIBGEN_H 1 */
 
@@ -281,7 +269,7 @@
 /* Define to 1 if you have the `localtime_r' function. */
 #define HAVE_LOCALTIME_R 1
 
-/* if your compiler supports long long */
+/* Define to 1 if the compiler supports the 'long long' data type. */
 #define HAVE_LONGLONG 1
 
 /* Define to 1 if you need the malloc.h header file even with stdlib.h  */
@@ -305,9 +293,6 @@
 /* Define if NI_WITHSCOPEID exists and works */
 /* #undef HAVE_NI_WITHSCOPEID */
 
-/* Defined if no inet_pton() prototype available */
-/* #undef HAVE_NO_INET_PTON_PROTO */
-
 /* we have no strerror_r() proto */
 /* #undef HAVE_NO_STRERROR_R_DECL */
 
@@ -602,12 +587,6 @@
 /* Define to the type of arg 5 for `select'. */
 #define SELECT_TYPE_ARG5 (struct timeval *)
 
-/* The size of a `curl_off_t', as computed by sizeof. */
-#define SIZEOF_CURL_OFF_T 8
-
-/* The size of a `long', as computed by sizeof. */
-#define SIZEOF_LONG 8
-
 /* The size of a `size_t', as computed by sizeof. */
 #define SIZEOF_SIZE_T 8
 
@@ -669,7 +648,7 @@
 /* Define to `unsigned' if <sys/types.h> does not define. */
 /* #undef size_t */
 
-/* type to use in place of socklen_t if not defined */
+/* Type to use in place of socklen_t when system does not provide it. */
 /* #undef socklen_t */
 
 /* the signed version of size_t */
@@ -711,6 +690,30 @@
 /* Define to the function return type for recv. */
 #define RECV_TYPE_RETV int
 
+/* Define to 1 if you have the recvfrom function. */
+#define HAVE_RECVFROM 1
+
+/* Define to the type of arg 1 for recvfrom. */
+#define RECVFROM_TYPE_ARG1 int
+
+/* Define to the type pointed by arg 2 for recvfrom. */
+#define RECVFROM_TYPE_ARG2 char
+
+/* Define to the type of arg 3 for recvfrom. */
+#define RECVFROM_TYPE_ARG3 int
+
+/* Define to the type of arg 4 for recvfrom. */
+#define RECVFROM_TYPE_ARG4 int
+
+/* Define to the type pointed by arg 5 for recvfrom. */
+#define RECVFROM_TYPE_ARG5 struct sockaddr
+
+/* Define to the type pointed by arg 6 for recvfrom. */
+#define RECVFROM_TYPE_ARG6 int
+
+/* Define to the function return type for recvfrom. */
+#define RECVFROM_TYPE_RETV int
+
 /* Define to 1 if you have the send function. */
 #define HAVE_SEND 1
 
@@ -737,15 +740,5 @@
 #define CURL_ICONV_CODESET_OF_HOST "IBM-1047"
 #endif
 
-/* ---------------------------------------------------------------- */
-/*                          HEADER FILES                            */
-/* ---------------------------------------------------------------- */
-
-#include <strings.h>    /* for bzero, strcasecmp, and strncasecmp */
-#include <string.h>     /* for strcpy and strlen */
-#include <stdlib.h>     /* for rand and srand */
-#include <sys/socket.h> /* for select and ioctl*/
-#include <netdb.h>      /* for in_addr_t definition */
-#include <tpf/sysapi.h> /* for tpf_process_signals */
 
 #endif /* __LIBCONFIGTPF_H */
