@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: getinfo.c,v 1.61 2008-05-12 21:43:29 bagder Exp $
+ * $Id: getinfo.c,v 1.65 2008-10-07 18:28:24 yangtse Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -78,6 +78,11 @@ CURLcode Curl_getinfo(struct SessionHandle *data, CURLINFO info, ...)
   struct curl_slist **param_slistp=NULL;
   int type;
 
+  union {
+    struct curl_certinfo * to_certinfo;
+    struct curl_slist    * to_slist;
+  } ptr;
+
   if(!data)
     return CURLE_BAD_FUNCTION_ARGUMENT;
 
@@ -137,6 +142,9 @@ CURLcode Curl_getinfo(struct SessionHandle *data, CURLINFO info, ...)
   case CURLINFO_CONNECT_TIME:
     *param_doublep = data->progress.t_connect;
     break;
+  case CURLINFO_APPCONNECT_TIME:
+    *param_doublep = data->progress.t_appconnect;
+    break;
   case CURLINFO_PRETRANSFER_TIME:
     *param_doublep =  data->progress.t_pretransfer;
     break;
@@ -210,6 +218,16 @@ CURLcode Curl_getinfo(struct SessionHandle *data, CURLINFO info, ...)
        option had been enabled! */
     *param_charp = data->info.wouldredirect;
     break;
+  case CURLINFO_PRIMARY_IP:
+    /* Return the ip address of the most recent (primary) connection */
+    *param_charp = data->info.ip;
+    break;
+  case CURLINFO_CERTINFO:
+    /* Return the a pointer to the certinfo struct. Not really an slist
+       pointer but we can pretend it is here */
+    ptr.to_certinfo = &data->info.certs;
+    *param_slistp = ptr.to_slist;
+    break;
   default:
     return CURLE_BAD_FUNCTION_ARGUMENT;
   }
