@@ -7,7 +7,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2005, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -20,18 +20,16 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.45 2006-08-04 01:13:24 yangtse Exp $
+ * $Id: setup.h,v 1.58 2008-09-05 01:27:24 yangtse Exp $
  ***************************************************************************/
 
 #define CURL_NO_OLDIES
 
-#if !defined(WIN32) && defined(__WIN32__)
-/* Borland fix */
-#define WIN32
-#endif
+/*
+ * Define WIN32 when build target is Win32 API
+ */
 
-#if !defined(WIN32) && defined(_WIN32)
-/* VS2005 on x64 fix */
+#if (defined(_WIN32) || defined(__WIN32__)) && !defined(WIN32) && !defined(__SYMBIAN32__)
 #define WIN32
 #endif
 
@@ -48,18 +46,22 @@
 #include "config-win32.h"
 #endif
 
-#ifdef macintosh
-#include "config-mac.h"
+#if defined(macintosh) && defined(__MRC__)
+#  include "config-mac.h"
 #endif
 
 #ifdef __riscos__
 #include "config-riscos.h"
 #endif
 
-#ifdef __amigaos__
+#ifdef __AMIGA__
 #include "config-amigaos.h"
 #endif
 
+#ifdef __SYMBIAN32__
+#include "config-symbian.h"
+#endif
+
 #ifdef TPF
 #include "config-tpf.h"
 /* change which select is used for the curl command line tool */
@@ -70,11 +72,16 @@
 
 #endif /* HAVE_CONFIG_H */
 
-#if defined(CURLDEBUG) && defined(CURLTOOLDEBUG)
-/* This is an ugly hack for CURLDEBUG conditions only. We need to include
-   the file here, since it might set the _FILE_OFFSET_BITS define, which must
-   be set BEFORE all normal system headers. */
-#include "../lib/setup.h"
+/*
+ * Tru64 needs _REENTRANT set for a few function prototypes and
+ * things to appear in the system header files. Unixware needs it
+ * to build proper reentrant code. Others may also need it.
+ */
+
+#ifdef NEED_REENTRANT
+#  ifndef _REENTRANT
+#    define _REENTRANT
+#  endif
 #endif
 
 /* 
@@ -105,23 +112,29 @@
 #  endif
 #endif
 
+/*
+ * Define USE_WINSOCK to 2 if we have and use WINSOCK2 API, else
+ * define USE_WINSOCK to 1 if we have and use WINSOCK  API, else
+ * undefine USE_WINSOCK.
+ */
+
+#undef USE_WINSOCK
+
+#ifdef HAVE_WINSOCK2_H
+#  define USE_WINSOCK 2
+#else
+#  ifdef HAVE_WINSOCK_H
+#    define USE_WINSOCK 1
+#  endif
+#endif
+
+
 #include <stdio.h>
 
 #ifdef __TANDEM
 #include <floss.h>
 #endif
 
-#ifndef TRUE
-#define TRUE 1
-#endif
-#ifndef FALSE
-#define FALSE 0
-#endif
-
-#if !defined(__cplusplus) && !defined(__BEOS__) && !defined(__ECOS) && !defined(typedef_bool)
-typedef unsigned char bool;
-#define typedef_bool
-#endif
 
 #ifndef OS
 #define OS "unknown"
@@ -170,8 +183,8 @@ int fileno( FILE *stream);
 #define typedef_bool
 #endif
 
-#ifndef SIZEOF_CURL_OFF_T
-#define SIZEOF_CURL_OFF_T sizeof(curl_off_t)
+#if (defined(NETWARE) && !defined(__NOVELL_LIBC__))
+#include <sys/timeval.h>
 #endif
 
 #ifndef UNPRINTABLE_CHAR
@@ -184,4 +197,12 @@ int fileno( FILE *stream);
 #define strdup(ptr) curlx_strdup(ptr)
 #endif
 
+/*
+ * Include macros and defines that should only be processed once.
+ */
+
+#ifndef __SETUP_ONCE_H
+#include "setup_once.h"
+#endif
+
 #endif /* __SRC_CURL_SETUP_H */
