@@ -5,11 +5,13 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: lib519.c,v 1.3 2006-10-25 09:20:44 yangtse Exp $
+ * $Id: lib519.c,v 1.5 2008-09-20 04:26:57 yangtse Exp $
  */
 
 #include "test.h"
 
+#include "memdebug.h"
+
 int test(char *URL)
 {
   CURLcode res;
@@ -28,8 +30,8 @@ int test(char *URL)
 
   curl_easy_setopt(curl, CURLOPT_URL, URL);
   curl_easy_setopt(curl, CURLOPT_USERPWD, "monster:underbed");
-  curl_easy_setopt(curl, CURLOPT_HEADER, TRUE);
-  curl_easy_setopt(curl, CURLOPT_VERBOSE, TRUE);
+  curl_easy_setopt(curl, CURLOPT_HEADER, 1L);
+  curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
 
   /* get first page */
   res = curl_easy_perform(curl);
