@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * $Id: test.h,v 1.16 2007-02-22 02:51:55 yangtse Exp $
+ * $Id: test.h,v 1.21 2008-09-18 16:21:09 yangtse Exp $
  */
 
 /* Now include the setup.h file from libcurl's private libdir (the source
@@ -15,7 +15,7 @@
 
 #include "setup.h"
 
-#include <curl.h>
+#include <curl/curl.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -36,10 +36,16 @@
 #define TEST_ERR_MAJOR_BAD     100
 #define TEST_ERR_RUNS_FOREVER   99
 
-extern char *arg2; /* set by first.c to the argv[2] or NULL */
+extern char *libtest_arg2; /* set by first.c to the argv[2] or NULL */
+extern char *libtest_arg3; /* set by first.c to the argv[3] or NULL */
 
-int select_test (int num_fds, fd_set *rd, fd_set *wr, fd_set *exc,
-                 struct timeval *tv);
+/* argc and argv as passed in to the main() function */
+extern int test_argc;
+extern char **test_argv;
+
+extern int select_test(int num_fds, fd_set *rd, fd_set *wr, fd_set *exc,
+                       struct timeval *tv);
+
+extern int test(char *URL); /* the actual test function provided by each
+                               individual libXXX.c file */
 
-int test(char *URL); /* the actual test function provided by each individual
-                        libXXX.c file */
