@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sws.c,v 1.122 2008-08-29 10:48:00 bagder Exp $
+ * $Id: sws.c,v 1.129 2008-10-01 17:34:25 danf Exp $
  ***************************************************************************/
 
 /* sws.c: simple (silly?) web server
@@ -71,14 +71,19 @@
 #define CURL_SWS_FORK_ENABLED
 #endif
 
+#ifdef ENABLE_IPV6
+static bool use_ipv6 = FALSE;
+#endif
+static const char *ipv_inuse = "IPv4";
+
 #define REQBUFSIZ 150000
 #define REQBUFSIZ_TXT "149999"
 
-long prevtestno=-1; /* previous test number we served */
-long prevpartno=-1; /* previous part number we served */
-bool prevbounce=FALSE; /* instructs the server to increase the part number for
-                          a test in case the identical testno+partno request
-                          shows up again */
+static long prevtestno=-1;    /* previous test number we served */
+static long prevpartno=-1;    /* previous part number we served */
+static bool prevbounce=FALSE; /* instructs the server to increase the part
+                                 number for a test in case the identical
+                                 testno+partno request shows up again */
 
 #define RCMD_NORMALREQ 0 /* default request, use the tests file normally */
 #define RCMD_IDLE      1 /* told to sit idle */
@@ -108,8 +113,8 @@ struct httprequest {
   bool pipelining; /* true if request is pipelined */
 };
 
-int ProcessRequest(struct httprequest *req);
-void storerequest(char *reqbuf, ssize_t totalsize);
+static int ProcessRequest(struct httprequest *req);
+static void storerequest(char *reqbuf, ssize_t totalsize);
 
 #define DEFAULT_PORT 8999
 
@@ -191,7 +196,7 @@ static void sigpipe_handler(int sig)
 }
 #endif
 
-int ProcessRequest(struct httprequest *req)
+static int ProcessRequest(struct httprequest *req)
 {
   char *line=&req->reqbuf[req->checkindex];
   bool chunked = FALSE;
@@ -273,7 +278,7 @@ int ProcessRequest(struct httprequest *req)
         error = ERRNO;
         logmsg("fopen() failed with error: %d %s", error, strerror(error));
         logmsg("Error opening file: %s", filename);
-        logmsg("Couldn't open test file %d", req->testno);
+        logmsg("Couldn't open test file %ld", req->testno);
         req->open = FALSE; /* closes connection */
         return 1; /* done */
       }
@@ -374,7 +379,7 @@ int ProcessRequest(struct httprequest *req)
    */
 
   do {
-    if((req->cl<=0) && curlx_strnequal("Content-Length:", line, 15)) {
+    if((req->cl==0) && curlx_strnequal("Content-Length:", line, 15)) {
       /* If we don't ignore content-length, we read it and we read the whole
          request including the body before we return. If we've been told to
          ignore the content-length, we will return as soon as all headers
@@ -382,9 +387,9 @@ int ProcessRequest(struct httprequest *req)
       size_t cl = strtol(line+15, &line, 10);
       req->cl = cl - req->skip;
 
-      logmsg("Found Content-Length: %d in the request", cl);
+      logmsg("Found Content-Length: %zu in the request", cl);
       if(req->skip)
-        logmsg("... but will abort after %d bytes", req->cl);
+        logmsg("... but will abort after %zu bytes", req->cl);
       break;
     }
     else if(curlx_strnequal("Transfer-Encoding: chunked", line,
@@ -418,16 +423,16 @@ int ProcessRequest(struct httprequest *req)
        Digest stuff to work in the test suite. */
     req->partno += 1000;
     req->digest = TRUE; /* header found */
-    logmsg("Received Digest request, sending back data %d", req->partno);
+    logmsg("Received Digest request, sending back data %ld", req->partno);
   }
   else if(!req->ntlm &&
           strstr(req->reqbuf, "Authorization: NTLM TlRMTVNTUAAD")) {
     /* If the client is passing this type-3 NTLM header */
     req->partno += 1002;
     req->ntlm = TRUE; /* NTLM found */
-    logmsg("Received NTLM type-3, sending back data %d", req->partno);
+    logmsg("Received NTLM type-3, sending back data %ld", req->partno);
     if(req->cl) {
-      logmsg("  Expecting %d POSTed bytes", req->cl);
+      logmsg("  Expecting %zu POSTed bytes", req->cl);
     }
   }
   else if(!req->ntlm &&
@@ -435,7 +440,7 @@ int ProcessRequest(struct httprequest *req)
     /* If the client is passing this type-1 NTLM header */
     req->partno += 1001;
     req->ntlm = TRUE; /* NTLM found */
-    logmsg("Received NTLM type-1, sending back data %d", req->partno);
+    logmsg("Received NTLM type-1, sending back data %ld", req->partno);
   }
   if(strstr(req->reqbuf, "Connection: close"))
     req->open = FALSE; /* close connection after this request */
@@ -482,10 +487,10 @@ int ProcessRequest(struct httprequest *req)
 }
 
 /* store the entire request in a file */
-void storerequest(char *reqbuf, ssize_t totalsize)
+static void storerequest(char *reqbuf, ssize_t totalsize)
 {
   int res;
-  int error;
+  int error = 0;
   ssize_t written;
   ssize_t writeleft;
   FILE *dump;
@@ -496,7 +501,7 @@ void storerequest(char *reqbuf, ssize_t totalsize)
   if (totalsize == 0)
     return;
   else if (totalsize < 0) {
-    logmsg("Invalid size (%d bytes) for request input. Not written to %s",
+    logmsg("Invalid size (%zd bytes) for request input. Not written to %s",
            totalsize, REQUEST_DUMP);
     return;
   }
@@ -522,7 +527,7 @@ void storerequest(char *reqbuf, ssize_t totalsize)
   if (writeleft > 0) {
     logmsg("Error writing file %s error: %d %s",
            REQUEST_DUMP, error, strerror(error));
-    logmsg("Wrote only (%d bytes) of (%d bytes) request input to %s",
+    logmsg("Wrote only (%zd bytes) of (%zd bytes) request input to %s",
            totalsize-writeleft, totalsize, REQUEST_DUMP);
   }
 
@@ -534,7 +539,7 @@ void storerequest(char *reqbuf, ssize_t totalsize)
            REQUEST_DUMP, error, strerror(error));
 
   if(!writeleft)
-    logmsg("Wrote request (%d bytes) input to " REQUEST_DUMP,
+    logmsg("Wrote request (%zd bytes) input to " REQUEST_DUMP,
            totalsize);
 }
 
@@ -545,7 +550,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
   char *reqbuf = req->reqbuf;
   ssize_t got = 0;
 
-  char *pipereq;
+  char *pipereq = NULL;
   int pipereq_length = 0;
 
   if(req->pipelining) {
@@ -575,7 +580,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
 
   while (req->offset < REQBUFSIZ-1) {
     if(pipereq_length) {
-      memmove(reqbuf, pipereq, pipereq_length); 
+      memmove(reqbuf, pipereq, pipereq_length);
       got = pipereq_length;
       pipereq_length = 0;
     }
@@ -601,7 +606,7 @@ static int get_request(curl_socket_t sock, struct httprequest *req)
       return DOCNUMBER_INTERNAL;
     }
 
-    logmsg("Read %d bytes", got);
+    logmsg("Read %zd bytes", got);
 
     req->offset += got;
     reqbuf[req->offset] = '\0';
@@ -650,14 +655,14 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
   bool persistant = TRUE;
   bool sendfailure = FALSE;
   size_t responsesize;
-  int error;
+  int error = 0;
   int res;
 
   static char weare[256];
 
   char partbuf[80]="data";
 
-  logmsg("Send response number %d part %d", req->testno, req->partno);
+  logmsg("Send response number %ld part %ld", req->testno, req->partno);
 
   switch(req->rcmd) {
   default:
@@ -695,7 +700,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
       logmsg("Identifying ourselves as friends");
       sprintf(msgbuf, "WE ROOLZ: %ld\r\n", (long)getpid());
       msglen = strlen(msgbuf);
-      sprintf(weare, "HTTP/1.1 200 OK\r\nContent-Length: %d\r\n\r\n%s",
+      sprintf(weare, "HTTP/1.1 200 OK\r\nContent-Length: %zu\r\n\r\n%s",
               msglen, msgbuf);
       buffer = weare;
       break;
@@ -795,7 +800,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
       break;
     }
     else {
-      logmsg("Sent off %d bytes", written);
+      logmsg("Sent off %zd bytes", written);
     }
     /* write to file as well */
     fwrite(buffer, 1, written, dump);
@@ -812,7 +817,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
            RESPONSE_DUMP, error, strerror(error));
 
   if(sendfailure) {
-    logmsg("Sending response failed. Only (%d bytes) of (%d bytes) were sent",
+    logmsg("Sending response failed. Only (%zu bytes) of (%zu bytes) were sent",
            responsesize-count, responsesize);
     if(ptr)
       free(ptr);
@@ -821,7 +826,7 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
     return -1;
   }
 
-  logmsg("Response sent (%d bytes) and written to " RESPONSE_DUMP,
+  logmsg("Response sent (%zu bytes) and written to " RESPONSE_DUMP,
          responsesize);
 
   if(ptr)
@@ -858,7 +863,6 @@ static int send_doc(curl_socket_t sock, struct httprequest *req)
   return 0;
 }
 
-bool use_ipv6=FALSE;
 
 int main(int argc, char *argv[])
 {
@@ -900,7 +904,8 @@ int main(int argc, char *argv[])
     }
     else if(!strcmp("--ipv6", argv[arg])) {
 #ifdef ENABLE_IPV6
-      use_ipv6=TRUE;
+      ipv_inuse = "IPv6";
+      use_ipv6 = TRUE;
 #endif
       arg++;
     }
@@ -986,13 +991,7 @@ int main(int argc, char *argv[])
     return 1;
   }
 
-  logmsg("Running IPv%d version on port %d",
-#ifdef ENABLE_IPV6
-         (use_ipv6?6:4)
-#else
-         4
-#endif
-         , port );
+  logmsg("Running %s version on port %d", ipv_inuse, (int)port);
 
   /* start accepting connections */
   rc = listen(sock, 5);
